@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2019 Inria.  All rights reserved.
+ * Copyright © 2009-2021 Inria.  All rights reserved.
  * Copyright © 2009-2011 Université Bordeaux
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright © 2011      Oracle and/or its affiliates.  All rights reserved.
@@ -467,7 +467,7 @@ lgrp_build_numanodes(struct hwloc_topology *topology,
 			      nids[i], obj->cpuset);
     }
 
-    hwloc_insert_object_by_cpuset(topology, obj);
+    hwloc__insert_object_by_cpuset(topology, NULL, obj, "solaris:lgrp:numa");
   }
   topology->support.discovery->numa = 1;
   topology->support.discovery->numa_memory = 1;
@@ -521,7 +521,7 @@ hwloc_look_lgrp(struct hwloc_topology *topology, struct hwloc_disc_status *dstat
             }
             distances[i*curlgrp+j] = (uint64_t) latency;
         }
-	hwloc_internal_distances_add(topology, "NUMA:Solaris", curlgrp, glob_lgrps, distances,
+	hwloc_internal_distances_add(topology, "NUMALatency", curlgrp, glob_lgrps, distances,
 				     HWLOC_DISTANCES_KIND_FROM_OS|HWLOC_DISTANCES_KIND_MEANS_LATENCY,
 				     HWLOC_DISTANCES_ADD_FLAG_GROUP);
 	glob_lgrps = NULL; /* dont free it below */
@@ -639,14 +639,15 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 
       if (kstat_read(kc, ksp, NULL) == -1)
 	{
-	  fprintf(stderr, "kstat_read failed for CPU%u: %s\n", cpuid, strerror(errno));
+          if (hwloc_hide_errors() < 2)
+            fprintf(stderr, "hwloc/solaris: kstat_read failed for CPU%u: %s\n", cpuid, strerror(errno));
 	  continue;
 	}
 
       hwloc_debug("cpu%u\n", cpuid);
       hwloc_bitmap_set(topology->levels[0][0]->complete_cpuset, cpuid);
 
-      stat = (kstat_named_t *) kstat_data_lookup(ksp, "state");
+      stat = (kstat_named_t *) kstat_data_lookup(ksp, (char *) "state");
       if (!stat)
           hwloc_debug("could not read state for CPU%u: %s\n", cpuid, strerror(errno));
       else if (stat->data_type != KSTAT_DATA_CHAR)
@@ -693,13 +694,15 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 
       if (look_chips) do {
 	/* Get Chip ID */
-	stat = (kstat_named_t *) kstat_data_lookup(ksp, "chip_id");
+	stat = (kstat_named_t *) kstat_data_lookup(ksp, (char *) "chip_id");
 	if (!stat)
 	  {
-	    if (Lpkg_num)
-	      fprintf(stderr, "could not read package id for CPU%u: %s\n", cpuid, strerror(errno));
-	    else
+	    if (Lpkg_num) {
+              if (hwloc_hide_errors() < 2)
+                fprintf(stderr, "hwloc/solaris: could not read package id for CPU%u: %s\n", cpuid, strerror(errno));
+            } else {
 	      hwloc_debug("could not read package id for CPU%u: %s\n", cpuid, strerror(errno));
+            }
 	    look_chips = 0;
 	    continue;
 	  }
@@ -719,7 +722,8 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	    break;
 #endif
 	  default:
-	    fprintf(stderr, "chip_id type %u unknown\n", (unsigned) stat->data_type);
+            if (hwloc_hide_errors() < 2)
+              fprintf(stderr, "hwloc/solaris: chip_id type %u unknown\n", (unsigned) stat->data_type);
 	    look_chips = 0;
 	    continue;
 	}
@@ -743,13 +747,15 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 
       if (look_cores) do {
 	/* Get Core ID */
-	stat = (kstat_named_t *) kstat_data_lookup(ksp, "core_id");
+	stat = (kstat_named_t *) kstat_data_lookup(ksp, (char *) "core_id");
 	if (!stat)
 	  {
-	    if (Lcore_num)
-	      fprintf(stderr, "could not read core id for CPU%u: %s\n", cpuid, strerror(errno));
-	    else
+	    if (Lcore_num) {
+              if (hwloc_hide_errors() < 2)
+                fprintf(stderr, "hwloc/solaris: could not read core id for CPU%u: %s\n", cpuid, strerror(errno));
+            } else {
 	      hwloc_debug("could not read core id for CPU%u: %s\n", cpuid, strerror(errno));
+            }
 	    look_cores = 0;
 	    continue;
 	  }
@@ -769,7 +775,8 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	    break;
 #endif
 	  default:
-	    fprintf(stderr, "core_id type %u unknown\n", (unsigned) stat->data_type);
+            if (hwloc_hide_errors() < 2)
+              fprintf(stderr, "hwloc/solaris: core_id type %u unknown\n", (unsigned) stat->data_type);
 	    look_cores = 0;
 	    continue;
 	}
@@ -797,10 +804,11 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 
     } else if (!strcmp("pg_hw_perf", ksp->ks_module)) {
       if (kstat_read(kc, ksp, NULL) == -1) {
-	fprintf(stderr, "kstat_read failed for module %s name %s instance %d: %s\n", ksp->ks_module, ksp->ks_name, ksp->ks_instance, strerror(errno));
+        if (hwloc_hide_errors() < 2)
+          fprintf(stderr, "hwloc/solaris: kstat_read failed for module %s name %s instance %d: %s\n", ksp->ks_module, ksp->ks_name, ksp->ks_instance, strerror(errno));
 	continue;
       }
-      stat = (kstat_named_t *) kstat_data_lookup(ksp, "cpus");
+      stat = (kstat_named_t *) kstat_data_lookup(ksp, (char *) "cpus");
       if (stat) {
 	hwloc_debug("found kstat module %s name %s instance %d cpus type %d\n", ksp->ks_module, ksp->ks_name, ksp->ks_instance, stat->data_type);
 	if (stat->data_type == KSTAT_DATA_STRING) {
@@ -816,7 +824,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	      l3->attr->cache.linesize = chip_info.cache_linesize[HWLOC_SOLARIS_CHIP_INFO_L3];
 	      l3->attr->cache.associativity = chip_info.cache_associativity[HWLOC_SOLARIS_CHIP_INFO_L3];
 	      l3->attr->cache.type = HWLOC_OBJ_CACHE_UNIFIED;
-	      hwloc_insert_object_by_cpuset(topology, l3);
+	      hwloc__insert_object_by_cpuset(topology, NULL, l3, "solaris:chipinfo:l3cache");
 	      cpuset = NULL; /* don't free below */
 	    }
 	  }
@@ -829,7 +837,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	      l2i->attr->cache.linesize = chip_info.cache_linesize[HWLOC_SOLARIS_CHIP_INFO_L2I];
 	      l2i->attr->cache.associativity = chip_info.cache_associativity[HWLOC_SOLARIS_CHIP_INFO_L2I];
 	      l2i->attr->cache.type = HWLOC_OBJ_CACHE_INSTRUCTION;
-	      hwloc_insert_object_by_cpuset(topology, l2i);
+	      hwloc__insert_object_by_cpuset(topology, NULL, l2i, "solaris:chipinfo:l2icache");
 	    }
 	    if (chip_info.cache_size[HWLOC_SOLARIS_CHIP_INFO_L2D] >= 0) {
 	      hwloc_obj_t l2 = hwloc_alloc_setup_object(topology, HWLOC_OBJ_L2CACHE, HWLOC_UNKNOWN_INDEX);
@@ -839,7 +847,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	      l2->attr->cache.linesize = chip_info.cache_linesize[HWLOC_SOLARIS_CHIP_INFO_L2D];
 	      l2->attr->cache.associativity = chip_info.cache_associativity[HWLOC_SOLARIS_CHIP_INFO_L2D];
 	      l2->attr->cache.type = chip_info.l2_unified ? HWLOC_OBJ_CACHE_UNIFIED : HWLOC_OBJ_CACHE_DATA;
-	      hwloc_insert_object_by_cpuset(topology, l2);
+	      hwloc__insert_object_by_cpuset(topology, NULL, l2, "solaris:chipinfo:l2cache");
 	      cpuset = NULL; /* don't free below */
 	    }
 	  }
@@ -850,7 +858,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	    group->attr->group.subkind = hwloc_bitmap_weight(cpuset);
 	    if (ksp->ks_name[0])
 	      hwloc_obj_add_info(group, "SolarisProcessorGroup", ksp->ks_name);
-	    hwloc_insert_object_by_cpuset(topology, group);
+	    hwloc__insert_object_by_cpuset(topology, NULL, group, "solaris:kstat:group");
 	    cpuset = NULL; /* don't free below */
 	  }
 	  hwloc_bitmap_free(cpuset);
@@ -875,7 +883,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	if (Pproc[k].Lpkg == j)
 	  hwloc_bitmap_set(obj->cpuset, k);
       hwloc_debug_1arg_bitmap("Package %u has cpuset %s\n", j, obj->cpuset);
-      hwloc_insert_object_by_cpuset(topology, obj);
+      hwloc__insert_object_by_cpuset(topology, NULL, obj, "solaris:kstat:package");
     }
     hwloc_debug("%s", "\n");
   }
@@ -905,7 +913,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	l1->attr->cache.size = chip_info.cache_size[HWLOC_SOLARIS_CHIP_INFO_L1D];
 	l1->attr->cache.linesize = chip_info.cache_linesize[HWLOC_SOLARIS_CHIP_INFO_L1D];
 	l1->attr->cache.associativity = chip_info.cache_associativity[HWLOC_SOLARIS_CHIP_INFO_L1D];
-	hwloc_insert_object_by_cpuset(topology, l1);
+	hwloc__insert_object_by_cpuset(topology, NULL, l1, "solaris:chipinfo:l1cache");
       }
       if (l1i_from_core) {
 	struct hwloc_obj *l1i = hwloc_alloc_setup_object(topology, HWLOC_OBJ_L1ICACHE, HWLOC_UNKNOWN_INDEX);
@@ -915,12 +923,12 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	l1i->attr->cache.size = chip_info.cache_size[HWLOC_SOLARIS_CHIP_INFO_L1I];
 	l1i->attr->cache.linesize = chip_info.cache_linesize[HWLOC_SOLARIS_CHIP_INFO_L1I];
 	l1i->attr->cache.associativity = chip_info.cache_associativity[HWLOC_SOLARIS_CHIP_INFO_L1I];
-	hwloc_insert_object_by_cpuset(topology, l1i);
+	hwloc__insert_object_by_cpuset(topology, NULL, l1i, "solaris:chipinfo:l1icache");
       }
       if (hwloc_filter_check_keep_object_type(topology, HWLOC_OBJ_CORE)) {
 	struct hwloc_obj *obj = hwloc_alloc_setup_object(topology, HWLOC_OBJ_CORE, Lcore[j].Pcore);
 	obj->cpuset = cpuset;
-	hwloc_insert_object_by_cpuset(topology, obj);
+	hwloc__insert_object_by_cpuset(topology, NULL, obj, "solaris:kstat:core");
       } else {
 	hwloc_bitmap_free(cpuset);
       }
@@ -939,7 +947,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	if (Pproc[k].Lproc == j)
 	  hwloc_bitmap_set(obj->cpuset, k);
       hwloc_debug_1arg_bitmap("PU %u has cpuset %s\n", j, obj->cpuset);
-      hwloc_insert_object_by_cpuset(topology, obj);
+      hwloc__insert_object_by_cpuset(topology, NULL, obj, "solaris:kstat:pu");
     }
     hwloc_debug("%s", "\n");
     topology->support.discovery->pu = 1;
